<div class="controls">
    <?= $tabs ?>
</div>
<div class="content">
    <?= $form->create()->setTitle(null) // @TODO(el): create() has to be called because the UserForm is setting the title there ?>
</div>
