<?php

use Icinga\Web\Navigation\Navigation;

$navigation = new Navigation();
$navigation->load($object->getType() . '-action');
foreach ($navigation as $item) {
    $item->setObject($object);
}

foreach ($object->getActionUrls() as $i => $link) {
    $navigation->addItem(

        // add warning to links that open in new tabs to improve accessibility, as recommended by WCAG20 G201
        $this->icon(
            'forward',
            $this->translate('Link opens in new window'),
            array('aria-label' => $this->translate('Link opens in new window'))
        ) . ' Action ' . ($i + 1),
        array(
            'url'       => $link,
            'target'    => '_blank',
            'renderer'  => array(
                'NavigationItemRenderer',
                'escape_label' => false
            )
        )
    );
}

if (isset($this->actions)) {
    $navigation->merge($this->actions);
}

if ($navigation->isEmpty() || ! $navigation->hasRenderableItems()) {
    return;
}

?>
<tr>
    <th><?= $this->translate('Actions'); ?></th>
    <?= $navigation->getRenderer()->setElementTag('td')->setCssClass('actions go-ahead'); ?>
</tr>
