<?php
/* Icinga Web 2 | (c) 2014 Icinga Development Team | GPLv2+ */

namespace Icinga\Module\Doc\Exception;

use Icinga\Exception\IcingaException;

/**
 * Exception thrown if an error in the documentation module's library occurs
 */
class DocException extends IcingaException
{
}
