<?php if (! $this->compact): ?>
<div class="controls">
    <?= $this->tabs->showOnlyCloseButton(); ?>
</div>
<?php endif ?>
<div class="content">
    <h1><?= $title; ?></h1>
    <?php if ((bool) $programStatus->notifications_enabled === false): ?>
    <div>
        <?= $this->translate('Host and service notifications are already disabled.') ?>
        <?php if ($this->programStatus->disable_notif_expire_time): ?>
            <?= sprintf(
                $this->translate('Notifications will be re-enabled in <strong>%s</strong>.'),
                $this->timeUntil($this->programStatus->disable_notif_expire_time)); ?>
        <?php endif; ?>
    </div>
    <?php else: ?>
    <?= $form; ?>
    <?php endif ?>
</div>
