<?php

if (! $this->compact): ?>
<div class="controls separated">
    <?= $this->tabs ?>
<?php
if ($this->header === true) {
    if ($object->type === 'service') {
        echo $this->render('partials/object/service-header.phtml');
    } else {
        echo $this->render('partials/object/host-header.phtml');
    }
} elseif ($this->header !== false) {
    echo $this->header;
}
?>
</div>
<?php endif ?>
<div class="content">
    <?= $this->content ?>
</div>