<?php

if ($object->is_flapping) {
    printf(
        "<tr><th>%s</th><td>%s %s</td></tr>\n",
        'Flapping',
        $this->icon('flapping', 'Flapping'),
        sprintf(
            'Currently flapping with a %.2f%% state change rate',
            $object->percent_state_change
        )
    );
}

