/*! Icinga Web 2 | (c) 2015 Icinga Development Team | GPLv2+ */

@icinga-blue: #2b95ff;
@control-color: @icinga-blue;

.letitsnow {
  background-image: url('../img/winter/snow1.png'), url('../img/winter/snow2.png'), url('../img/winter/snow3.png');
  animation: ~"snow" 10s linear infinite;
}

#header-logo {
    background-image: url('../img/winter/logo_icinga_big_winter.png');
}

/* Snow, from http://codepen.io/NickyCDK/pen/AIonk */
#login, #header-logo-container, #main > .container > .controls > .tabs {
  .letitsnow()
}

@keyframes ~"snow" {
  0% {background-position: 0px 0px, 0px 0px, 0px 0px;}
  50% {background-position: 500px 500px, 100px 200px, -100px 150px;}
  100% {background-position: 500px 1000px, 200px 400px, -100px 300px;}
}

#menu ul.nav-level-1 > .nav-item {
  &:focus, &:hover {
    .letitsnow()
  }
}
