<div class="controls controls-separated">
<?php if (! $this->compact): ?>
    <?= $this->tabs ?>
<?php endif ?>
    <?= $this->render('partials/object/service-header.phtml') ?>
    <?= $this->render('partials/object/quick-actions.phtml') ?>
</div>
<?= $this->render('partials/object/detail-content.phtml') ?>
