<?php use Icinga\Data\Filter\Filter; ?>

<div class="controls">
  <?php if (! $this->compact): ?>
    <?= $this->tabs; ?>
  <?php endif ?>
  <?= $this->render('partials/object/host-header.phtml') ?>
    <?php
      $this->baseFilter = Filter::where('host', $object->host_name);
      $this->stats = $object->stats;
      echo $this->render('list/components/servicesummary.phtml');
    ?>
</div>
<?= $this->partial(
  'list/services.phtml',
  'monitoring',
  array(
    'compact'       => true,
    'showHost'      => false,
    'services'      => $services,
    'addColumns'    => array()
  )
); ?>
