<div class="content">
    <h1><?= $this->translate('Could not save dashboard'); ?></h1>
    <p>
        <?= $this->translate('Please copy the following dashboard snippet to '); ?>
        <strong><?= $this->config->getConfigFile(); ?>;</strong>.
        <br>
        <?= $this->translate('Make sure that the webserver can write to this file.'); ?>
    </p>
    <pre><?= $this->config; ?></pre>
    <hr>
    <h2><?= $this->translate('Error details'); ?></h2>
    <p><?= $this->error->getMessage(); ?></p>
</div>