<?php

if (empty($object->hostgroups)) return;

$list = array();
foreach ($object->hostgroups as $name => $alias) {
    $list[] = $this->qlink(
        $alias,
        'monitoring/list/hosts',
        array('hostgroup_name' => $name),
        array('title' => sprintf($this->translate('List all hosts in the group "%s"'), $alias))
    );
}
printf(
    "<tr><th>%s</th><td class=\"go-ahead\">%s</td></tr>\n",
    $this->translate('Hostgroups'),
    implode(', ', $list)
);

